-- Update by Silentdave (Ravegrobber - Kul'Tiras EU)
-- Due to various XML attribute changes in 3.0, the auto show and timer controlled hide of the PoisonerMenu no longer works. 
-- Thus added OnMouseUp to Minimapbutton in the XML file to toggle PoisonerMenu showing/hiding (linked to PoisonerMenu_Toggle() in this file

-- OnLoad event
function Poisoner_OnLoad(self)
	-- register for clicks
	self:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	self:RegisterForDrag("LeftButton");
	-- need this for the initial load
	self:RegisterEvent("VARIABLES_LOADED");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
      -- register a slash command
        SLASH_POISONER1 = "/poisoner";
        SlashCmdList["POISONER"] = function(msg)
                Poisoner_ProcessCommand(msg);
        end;
	
	Poisoner_ReapplyButtonVisible = 0;
	
	StaticPopupDialogs["POISONER_QUESTION_ENABLE"] = {
		text = "|cff00ff00Poisoner:|r\n Your character is not a rogue.\n Enable Poisoner anyway?",
		button1 = YES,
		button2 = NO,
		OnAccept = function() Poisoner_Enable()  end,
		timeout = 0,
		whileDead = 0,
		hideOnEscape = 1
	}
	
	StaticPopupDialogs["POISONER_STARTED_ONCE"] = {
		text = "|cff00ff00Poisoner:|r\n"..OPTION_RESTART_REQUIREMENT.."",
		button1 = OKAY,
		button2 = CANCEL,
		OnAccept = function() Poisoner_RestartOnce()  end,
		timeout = 0,
		whileDead = 0,
		hideOnEscape = 1
	}
	
	PoisonerDebug("Poisoner Loaded");

end

function Poisoner_OnEvent(self, event)
	if (event == "VARIABLES_LOADED") then
		Poisoner_OnVariablesLoaded(self);
	elseif (event == "PLAYER_REGEN_DISABLED") then
		Poisoner_OnEnterCombat();
	elseif (event == "PLAYER_REGEN_ENABLED") then
		Poisoner_OnLeaveCombat();
	end
end

function Poisoner_OnVariablesLoaded(self)
	if (POISONER_CONFIG == nil) then
		POISONER_CONFIG = {};
		local _, cls = UnitClass("player");
		if (cls == "ROGUE") then
			POISONER_CONFIG.Enabled = 1;
		elseif not (cls == "ROGUE") then
			StaticPopup_Show("POISONER_QUESTION_ENABLE");
		end
	end
	if ( not POISONER_CONFIG.Enable ) then
		local _, cls = UnitClass("player");
		if (cls == "ROGUE") then
			POISONER_CONFIG.Enabled = 1;
		elseif not (cls == "ROGUE") then
			StaticPopup_Show("POISONER_QUESTION_ENABLE");
		end
	end
	if ( not POISONER_CONFIG.StartedOnce ) then
		POISONER_CONFIG.StartedOnce = 0;
	end
	if (POISONER_CONFIG.StartedOnce == 0) and (POISONER_CONFIG.Enabled == 1) then
		StaticPopup_Show("POISONER_STARTED_ONCE");
	end
	if (POISONER_CONFIG.Scale ~= nil) then
		PoisonerMenu:SetScale(POISONER_CONFIG.Scale);
	end
	Poisoner_SetPosition(POISONER_CONFIG.MinimapButton.MenuPosition or "left");
	if (POISONER_CONFIG.Enabled == 1) then
		PoisonerStateHeader:Show();
		if (POISONER_CONFIG.MinimapButton.Active == 1) then
			PoisonerMinimapButton:Show();
		elseif (POISONER_CONFIG.MinimapButton.Active == 0) then
			PoisonerMinimapButton:Hide();
		end
	else
		PoisonerStateHeader:Hide();
		PoisonerMinimapButton:Hide();
	end
	if (not POISONER_CONFIG.TooltipType) then
		POISONER_CONFIG.TooltipType = "full"
	end
	
	PoisonerStateHeader.inCombat = false;
	PoisonerStateHeader:SetAttribute("state", "0");
	self:UnregisterEvent("VARIABLES_LOADED");
end

-- Restart once to update locales in SavedVariables
function Poisoner_RestartOnce()
	Poisoner_GetPoisonNames();
	POISONER_CONFIG.StartedOnce = 1;
	ReloadUI();
end

-- when entering combat we need to pregenerate the buttons, as that cannot be done during lockdown
function Poisoner_OnEnterCombat()
	PoisonerDebug("entering combat");
	PoisonerStateHeader.poisonCount = Poisoner_FindPoisons();
	Poisoner_CreateButtons();
	PoisonerStateHeader.inCombat = true;
end

function Poisoner_OnLeaveCombat()
	PoisonerDebug("leaving combat");
	PoisonerStateHeader.inCombat = false;
end

function Poisoner_GetPoisonNames()

	POISONER_CONFIG.Poisons = { 
		CPName = GetItemInfo(3775);	--Verkrppelndes Gift
		MPName = GetItemInfo(5237);	--Gedankenbenebelndes Gift
		DPName = GetItemInfo(2892);	--Tdliches Gift
		IPName = GetItemInfo(6947);	--Sofort wirkendes Gift
		WPName = GetItemInfo(10918);	--Wundgift
	}

end

function Poisoner_OnItemButtonUpdate(button, elapsed)
	button.elapsed = button.elapsed + elapsed;
	if (button.needUpdate or button.elapsed > button.updateDelay) then
		-- find the current poison count for this button
		local _, _, itemName = string.find(button:GetName(), "^PoisonerMenuButton(.*)$");
		local count, rank;
		if (PoisonerStateHeader.poisonCount) then
			local rec = PoisonerStateHeader.poisonCount[itemName];
			if (rec) then
				count = rec.quantity;
				rank = rec.rank;
			end
		end
		
		-- and display it
		-- local cbutton = _G[button:GetName()];
		if (count and count > 0) then
			button:SetText(""..count);
			button:GetNormalTexture():SetVertexColor(1,1,1,1);
		else
			button:SetText("0");
			button:GetNormalTexture():SetVertexColor(0.5,0.5,0.5,1);
			button:GetHighlightTexture():SetVertexColor(0.5,0.5,0.5,1);
		end
		button.elapsed = 0;
		button.needUpdate = false;
	end
	
	Poisoner_CheckReapplyButton()
	
end

function Poisoner_OnMenuShow()
	-- find the current number of poisons available
	PoisonerStateHeader.poisonCount = Poisoner_FindPoisons();
	if (not PoisonerStateHeader.inCombat) then
		-- generate the appropriate buttons
		Poisoner_CreateButtons();
      end
end

function Poisoner_OnItemButtonClick(button, click)
	local _, _, itemName = string.find(button:GetName(), "^PoisonerMenuButton(.*)$");
	if (not SpellCanTargetItem()) then
		-- Poison is applying, decrement the count
		if (PoisonerStateHeader.poisonCount) then
			local rec = PoisonerStateHeader.poisonCount[itemName];
			if (rec) then
				rec.quantity = rec.quantity - 1;
			end
		end
	else
		-- still targetting, the box was displayed
		-- TODO: need to clear the cursor here somehow
	end
	button.needUpdate = true;
	
				local COLOR_GREEN = "|cff00ff00";
				local COLOR_END = "|r";
	
				if (click=="LeftButton") then
					Poisoner_Poison16.Name = itemName
						local weaponID = GetInventoryItemID("player", 16)
						local weaponName = GetItemInfo(weaponID)
						print(""..INVTYPE_WEAPONMAINHAND..": "..weaponName.." > "..COLOR_GREEN..Poisoner_Poison16.Name..COLOR_END)
				elseif (click=="RightButton") then
					Poisoner_Poison17.Name = itemName
						local weaponID = GetInventoryItemID("player", 17)
						local weaponName = GetItemInfo(weaponID)
						print(""..INVTYPE_WEAPONOFFHAND..": "..weaponName.." > "..COLOR_GREEN..Poisoner_Poison17.Name..COLOR_END)
				elseif (click=="MiddleButton") then
					Poisoner_Poison18.Name = itemName
						local weaponID = GetInventoryItemID("player", 18)
						local weaponName = GetItemInfo(weaponID)
						print(""..INVTYPE_THROWN..": "..weaponName.." > "..COLOR_GREEN..Poisoner_Poison18.Name..COLOR_END)
				else
					print("No click registered")
				end
				
end


-- This function creates/initialises the individual buttons for each poison
-- type. Note that we re-use buttons if they had been previously created, as
-- there is no delete operation for frames.
function Poisoner_CreateButtons()
	local menu = PoisonerMenu;
	local i = 0;
	PoisonerDebug("Children of PoisonerMenu:")
	local children = { menu:GetChildren() };
	for _, child in ipairs(children) do
		PoisonerDebug(child:GetName());
		i = i+1;
	end
	PoisonerDebug(i.." children");

	local x = 0;
	for _, t in pairs(Poisoner_FindPoisons()) do
		local buttonName = "PoisonerMenuButton"..t.name;
		local button = _G[buttonName];
		if (not PoisonerStateHeader.inCombat) then
			if (button == nil) then

				button = CreateFrame("Button", buttonName, menu, "PoisonerItemButtonTemplate");
				local bfs = button:CreateFontString(buttonName.."Text","OVERLAY","GameFontNormal");
				bfs:SetPoint("BOTTOMRIGHT");
				bfs:SetTextColor(1,1,1,1);
				bfs:SetShadowColor(0,0,0,1);
				bfs:SetShadowOffset(-2,2);
				button:SetFontString(bfs);

				button:SetNormalTexture(t.icon);
				button:SetHighlightTexture(t.icon);
				button:GetNormalTexture():SetVertexColor(1,1,1,1);
				button:GetHighlightTexture():SetVertexColor(0.5,0.5,1,1);

				button:SetAttribute("type", "macro");
				button:SetAttribute("macrotext", "/use "..t.name.."\n/use [button:3] 18\n/use [button:2] 17\n/use [button:1] 16\n/click StaticPopup1Button1");
				

			end
			button:Hide();
			button:ClearAllPoints();
			if (t.quantity and t.quantity>0) then
				button:SetScript("OnEnter", function() Poisoner_Tooltip(button,t.bag,t.bagslot,t.name) end);	
				button:SetScript("OnLeave", function() GameTooltip:Hide() end);
				if (POISONER_CONFIG.MinimapButton.MenuPosition == "right") then
					button:SetPoint("TOPLEFT", x, 0);
					x = x + button:GetWidth() + 4;
				else
					button:SetPoint("TOPRIGHT", x, 0);
					x = x - button:GetWidth() - 4;
				end
				button:Show();
			end
			
			button.needUpdate = true;
		end
	end
	
	--[[local testbuttonName = PoisonerTestButton;
	if (testbutton == nil) then
	testbutton = CreateFrame("Button", "PoisonerTestButton", menu, "PoisonerItemButtonTemplate2");

				testbutton:SetNormalTexture("Interface\\Icons\\Spell_Nature_Reincarnation");
				testbutton:SetHighlightTexture("Interface\\Icons\\Spell_Nature_Reincarnation");
				testbutton:GetNormalTexture():SetVertexColor(1,1,1,1);
				testbutton:GetHighlightTexture():SetVertexColor(0.5,0.5,1,1);

				testbutton:SetAttribute("type", "macro");
				testbutton:SetAttribute("macrotext", "/use Sofort wirkendes Gift IX\n/use [button:2] 17\n/use [button:1] 16\n/click StaticPopup1Button1\n/say Poisoner TEST");
				
			--testbutton:Hide();
			testbutton:ClearAllPoints();
			testbutton:Show();
			if (POISONER_CONFIG.MinimapButton.MenuPosition == "right") then
				testbutton:SetPoint("TOPLEFT", x, 0);
				x = x + button:GetWidth() + 4;
			else
				testbutton:SetPoint("TOPRIGHT", x, 0);
				x = x - button:GetWidth() - 4;
			end
			testbutton:Show();
	
	end
	]]
end

--
-- Find available poisons
-- Returns an table keyed by poison name, value a table containing texture and quantity
--
function Poisoner_FindPoisons()
	local i, itemLink, bagSlots, bagId, icon, quantity;
	local ret = {};
	for i=0, 4 do
		itemLink = nil;
		bagSlots = GetContainerNumSlots(i);
		if (bagSlots > 0) then
			for j=1, bagSlots do
				itemLink = nil;
				itemLink = GetContainerItemLink(i, j);
				icon, quantity = GetContainerItemInfo(i, j)
				if (itemLink) then
					local itemId = nil;
					local itemName = nil;
					local itemRank = nil;
					local idx,pattern;
					for idx, pattern in ipairs(Poisoner_Patterns) do
						_, _, itemId, itemName, itemRank = string.find(itemLink, "item:(%d+).*%[("..pattern..")%]");
						if (itemName ~= nil) then
							break;
						end
					end
					
					if (itemName ~= nil) then
						-- if the quantity is 1, we either have a single consumable item, or an expendable item
						if (quantity == 1) then
							local charges = Poisoner_CountCharges(i, j);
							if (charges ~= nil) then
								quantity = charges;
							end
						end

						local t = ret[itemName];
						if (t == nil) then
							t = { name=itemName, icon=icon, quantity=0, rank=itemRank, id=itemId, bag=i, bagslot=j };
							ret[itemName] = t;
						end
						t.quantity = t.quantity + quantity;
						Poisoner_PoisonsEverSeen[itemName] = icon;
					end
				end
			end
		end
	end
	-- include zero counts for previously seen items we no longer have
	for prevItemName, prevIcon in pairs(Poisoner_PoisonsEverSeen) do
		local t = ret[prevItemName];
		if (t == nil) then
			t = { name=prevItemName, icon=prevIcon, quantity=0, bag=-1, bagslot=-1 };
			ret[prevItemName] = t;
		end
	end
	return ret;
end
-- since we cannot destroy frames, we must always remember everything we've seen
Poisoner_PoisonsEverSeen = {};

-- Use a tooltip to scan for the number of 'charges' on expendable items.
-- Returns the number of charges, or nil if it could not be found.
function Poisoner_CountCharges(bagId, bagSlot)
PoisonerDebug("Looking for charges of "..bagId..","..bagSlot);
	local ret = nil;
	PoisonerScannerTooltip:SetOwner(UIParent, "ANCHOR_NONE");
	PoisonerScannerTooltip:SetBagItem(bagId, bagSlot);
	for i=1,PoisonerScannerTooltip:NumLines() do
		local field = _G["PoisonerScannerTooltipTextLeft"..i];
		if (field ~= nil) then
			local text = field:GetText();
			if (text) then
				local _, _, charges = string.find(text, POISONER_PATTERN_CHARGES);
				if (charges) then
					ret = tonumber(charges);
					break;
				end
			end
		end
	end
	PoisonerScannerTooltip:Hide();
	return ret;
end


--
-- Quick reapply button
--

function Poisoner_CheckReapplyButton()

	if (POISONER_CONFIG.Enabled == 1) then
		if ((Poisoner_Poison16.Name ~= nil and Poisoner_Poison16.Name ~= "") and (Poisoner_Poison17.Name ~= nil and Poisoner_Poison17.Name ~= "") and (Poisoner_Poison18.Name ~= nil and Poisoner_Poison18.Name ~= "")) then
			if (Poisoner_ReapplyButtonVisible == 0) then
				Poisoner_CreateReapplyButton()
			elseif (Poisoner_ReapplyButtonVisible == 1) then
				Poisoner_UpdateReapplyButton()
			end
		elseif ((Poisoner_Poison16.Name == nil or Poisoner_Poison16.Name == "") and (Poisoner_Poison17.Name == nil or Poisoner_Poison17.Name == "") and (Poisoner_Poison18.Name == nil or Poisoner_Poison18.Name == "")) then
			Poisoner_ReapplyButton:Hide();
		end
	elseif (POISONER_CONFIG.Enabled == 0) then
		Poisoner_ReapplyButton:Hide();
	end
	
end

function Poisoner_GetToolTipTexture()

	local CPName = POISONER_CONFIG.Poisons.CPName		--Verkrppelndes Gift
	local MPName = POISONER_CONFIG.Poisons.MPName		--Gedankenbenebelndes Gift
	local DPName = POISONER_CONFIG.Poisons.DPName		--Tdliches Gift
	local IPName = POISONER_CONFIG.Poisons.IPName		--Sofort wirkendes Gift
	local WPName = POISONER_CONFIG.Poisons.WPName		--Wundgift
	
	local NoTexture = ""	--"Interface\\Icons\\INV_Misc_QuestionMark"
	local CPTexture = GetItemIcon(3775)
	local MPTexture = GetItemIcon(5237)
	local DPTexture = GetItemIcon(2892)
	local IPTexture = GetItemIcon(6947)
	local WPTexture = GetItemIcon(10918)

	Poisoner_PoisonSlots = {
		Poisoner_Poison16,
		Poisoner_Poison16_SHIFT,
		Poisoner_Poison16_CTRL,
		Poisoner_Poison16_ALT,
		Poisoner_Poison17,
		Poisoner_Poison17_SHIFT,
		Poisoner_Poison17_CTRL,
		Poisoner_Poison17_ALT,
		Poisoner_Poison18,
		Poisoner_Poison18_SHIFT,
		Poisoner_Poison18_CTRL,
		Poisoner_Poison18_ALT,
	}

	for k,v in pairs(Poisoner_PoisonSlots) do
		if v ~= nil then
			if (not v.Name) or (v.Name == nil) then
				v.Texture = NoTexture
				v.Name = ""
			elseif v.Name == CPName then
				v.Texture = CPTexture
			elseif v.Name == MPName then
				v.Texture = MPTexture
			elseif v.Name == DPName then
				v.Texture = DPTexture
			elseif v.Name == IPName then
				v.Texture = IPTexture
			elseif v.Name == WPName then
				v.Texture = WPTexture
			elseif v.Name == "" then
				v.Texture = NoTexture
			end
		else
			v.Texture = NoTexture
			v.Name = ""
		end
	end
	
end

function Poisoner_CreateReapplyButton()

		--local btn = CreateFrame("Button", "Poisoner_ReapplyButton", UIParent, "PoisonerReapplyTemplate");
		local btn = Poisoner_ReapplyButton
		
		btn:RegisterForClicks("LeftButtonDown", "RightButtonDown", "MiddleButtonDown");
		btn:SetNormalTexture("Interface\\Icons\\Ability_Creature_Poison_06");
		btn:SetHighlightTexture("Interface\\Icons\\Ability_Creature_Poison_06");
		btn:GetNormalTexture():SetVertexColor(1,1,1,1);
		btn:GetHighlightTexture():SetVertexColor(0.5,0.5,1,1);
		
		Poisoner_GetToolTipTexture()
		
		btn:SetAttribute("type", "macro");
		--btn:SetAttribute("macrotext", "/use [button:3] "..Poisoner_Poison18.Name.."\n/use [button:3] 18\n/use [button:2] "..Poisoner_Poison17.Name.."\n/use [button:2] 17\n/use [button:1] "..Poisoner_Poison16.Name.."\n/use [button:1] 16\n/click StaticPopup1Button1");
		btn:SetAttribute("macrotext", "/use [nomodifier,button:3] "..Poisoner_Poison18.Name.."\n/use [modifier:shift,button:3] "..Poisoner_Poison18_SHIFT.Name.."\n/use [modifier:ctrl,button:3] "..Poisoner_Poison18_CTRL.Name.."\n/use [modifier:alt,button:3] "..Poisoner_Poison18_ALT.Name.."\n/use [button:3] 18\n/use [nomodifier,button:2] "..Poisoner_Poison17.Name.."\n/use [modifier:shift,button:2] "..Poisoner_Poison17_SHIFT.Name.."\n/use [modifier:ctrl,button:2] "..Poisoner_Poison17_CTRL.Name.."\n/use [modifier:alt,button:2] "..Poisoner_Poison17_ALT.Name.."\n/use [button:2] 17\n/use [nomodifier,button:1] "..Poisoner_Poison16.Name.."\n/use [modifier:shift,button:1] "..Poisoner_Poison16_SHIFT.Name.."\n/use [modifier:ctrl,button:1] "..Poisoner_Poison16_CTRL.Name.."\n/use [modifier:alt,button:1] "..Poisoner_Poison16_ALT.Name.."\n/use [button:1] 16\n/click StaticPopup1Button1");
		
		btn:SetScript("OnEnter", function() Poisoner_ReapplyButton_Tooltip(btn) end);	
		btn:SetScript("OnLeave", function() GameTooltip:Hide() end);
		
		btn:ClearAllPoints();
		btn:SetPoint("CENTER", Poisoner_ReapplyButton_Overlay, "CENTER");
		btn:Show();
		Poisoner_ReapplyButtonVisible = 1;
	
	-- Tooltip layout
		
	local COLOR_GREEN = "|cff00ff00";
	local COLOR_GREY = "|cff808080";
	local COLOR_END = "|r";
		
	--Poisoner_ReapplyButton_Text = ""..COLOR_GREY..Poisoner_TooltipTitle..COLOR_END.."\n"..INVTYPE_WEAPONMAINHAND..": "..COLOR_GREEN..Poisoner_Poison16.Name..COLOR_END.."\n"..INVTYPE_WEAPONOFFHAND..": "..COLOR_GREEN..Poisoner_Poison17.Name..COLOR_END.."\n"..INVTYPE_THROWN..": "..COLOR_GREEN..Poisoner_Poison18.Name..COLOR_END..""
	
	Poisoner_ReapplyButton_Text_MH = "|T"..Poisoner_Poison16.Texture..":0|t |T"..Poisoner_Poison16_SHIFT.Texture..":0|t |T"..Poisoner_Poison16_CTRL.Texture..":0|t |T"..Poisoner_Poison16_ALT.Texture..":0|t "
	Poisoner_ReapplyButton_Text_OH = "|T"..Poisoner_Poison17.Texture..":0|t |T"..Poisoner_Poison17_SHIFT.Texture..":0|t |T"..Poisoner_Poison17_CTRL.Texture..":0|t |T"..Poisoner_Poison17_ALT.Texture..":0|t "
	Poisoner_ReapplyButton_Text_TW = "|T"..Poisoner_Poison18.Texture..":0|t |T"..Poisoner_Poison18_SHIFT.Texture..":0|t |T"..Poisoner_Poison18_CTRL.Texture..":0|t |T"..Poisoner_Poison18_ALT.Texture..":0|t "
	
	Poisoner_ReapplyButton_Text_Slots = ""..INVTYPE_WEAPONMAINHAND..": \n"..INVTYPE_WEAPONOFFHAND..": \n"..INVTYPE_THROWN..": "
	Poisoner_ReapplyButton_Text_Textures = ""..Poisoner_ReapplyButton_Text_MH.."\n"..Poisoner_ReapplyButton_Text_OH.."\n"..Poisoner_ReapplyButton_Text_TW..""
	
	Poisoner_ReapplyButton:Show();
		
end

function Poisoner_UpdateReapplyButton()
	local btn = Poisoner_ReapplyButton
	btn:SetAttribute("type", "macro");
	--btn:SetAttribute("macrotext", "/use [button:3] "..Poisoner_Poison18.Name.."\n/use [button:3] 18\n/use [button:2] "..Poisoner_Poison17.Name.."\n/use [button:2] 17\n/use [button:1] "..Poisoner_Poison16.Name.."\n/use [button:1] 16\n/click StaticPopup1Button1");
	btn:SetAttribute("macrotext", "/use [nomodifier,button:3] "..Poisoner_Poison18.Name.."\n/use [modifier:shift,button:3] "..Poisoner_Poison18_SHIFT.Name.."\n/use [modifier:ctrl,button:3] "..Poisoner_Poison18_CTRL.Name.."\n/use [modifier:alt,button:3] "..Poisoner_Poison18_ALT.Name.."\n/use [button:3] 18\n/use [nomodifier,button:2] "..Poisoner_Poison17.Name.."\n/use [modifier:shift,button:2] "..Poisoner_Poison17_SHIFT.Name.."\n/use [modifier:ctrl,button:2] "..Poisoner_Poison17_CTRL.Name.."\n/use [modifier:alt,button:2] "..Poisoner_Poison17_ALT.Name.."\n/use [button:2] 17\n/use [nomodifier,button:1] "..Poisoner_Poison16.Name.."\n/use [modifier:shift,button:1] "..Poisoner_Poison16_SHIFT.Name.."\n/use [modifier:ctrl,button:1] "..Poisoner_Poison16_CTRL.Name.."\n/use [modifier:alt,button:1] "..Poisoner_Poison16_ALT.Name.."\n/use [button:1] 16\n/click StaticPopup1Button1");
		
	Poisoner_GetToolTipTexture()
	
	-- Tooltip layout
		
	local COLOR_GREEN = "|cff00ff00";
	local COLOR_GREY = "|cff808080";
	local COLOR_END = "|r";
		
	--Poisoner_ReapplyButton_Text = ""..COLOR_GREY..Poisoner_TooltipTitle..COLOR_END.."\n"..INVTYPE_WEAPONMAINHAND..": "..COLOR_GREEN..Poisoner_Poison16.Name..COLOR_END.."\n"..INVTYPE_WEAPONOFFHAND..": "..COLOR_GREEN..Poisoner_Poison17.Name..COLOR_END.."\n"..INVTYPE_THROWN..": "..COLOR_GREEN..Poisoner_Poison18.Name..COLOR_END..""
	
	Poisoner_ReapplyButton_Text_MH = "|T"..Poisoner_Poison16.Texture..":0|t |T"..Poisoner_Poison16_SHIFT.Texture..":0|t |T"..Poisoner_Poison16_CTRL.Texture..":0|t |T"..Poisoner_Poison16_ALT.Texture..":0|t "
	Poisoner_ReapplyButton_Text_OH = "|T"..Poisoner_Poison17.Texture..":0|t |T"..Poisoner_Poison17_SHIFT.Texture..":0|t |T"..Poisoner_Poison17_CTRL.Texture..":0|t |T"..Poisoner_Poison17_ALT.Texture..":0|t "
	Poisoner_ReapplyButton_Text_TW = "|T"..Poisoner_Poison18.Texture..":0|t |T"..Poisoner_Poison18_SHIFT.Texture..":0|t |T"..Poisoner_Poison18_CTRL.Texture..":0|t |T"..Poisoner_Poison18_ALT.Texture..":0|t "
	
	Poisoner_ReapplyButton_Text_Slots = ""..INVTYPE_WEAPONMAINHAND..": \n"..INVTYPE_WEAPONOFFHAND..": \n"..INVTYPE_THROWN..": "
	Poisoner_ReapplyButton_Text_Textures = ""..Poisoner_ReapplyButton_Text_MH.."\n"..Poisoner_ReapplyButton_Text_OH.."\n"..Poisoner_ReapplyButton_Text_TW..""
	
	Poisoner_ReapplyButton:Show();
end


--
-- Slash command processing
--
function Poisoner_ProcessCommand(command)
	if (command == "lock") then
		Poisoner_DragLock();
	elseif (command == "unlock") then
		Poisoner_DragUnlock();
	elseif (command == "enable") then
		Poisoner_Enable();
	elseif (command == "disable") then
		Poisoner_Disable();
	elseif (command == "reset") then
		Poisoner_DragReset();
		Poisoner_SetScale(1.0);
		Poisoner_SetPosition("left");
	elseif (string.match(command, "^scale")) then
		local arg = string.match(command, "^scale%s+([%d%.]+)");
		Poisoner_SetScale(arg);
	elseif (string.match(command, "^position")) then
		local arg = string.match(command, "^position%s(%w+)");
		Poisoner_SetPosition(arg);
	elseif (command == "name") then
		POISONER_CONFIG.TooltipType = "name";
	elseif (command == "full") then
		POISONER_CONFIG.TooltipType = "full";
	elseif (command == "config") then
		PoisonerOptions_ToggleSettings();
	else
		DEFAULT_CHAT_FRAME:AddMessage(POISONER_COMMANDERROR..command, 1.0, 1.0, 0.5);
	end
end

function Poisoner_Disable()
	POISONER_CONFIG.Enabled = 0;
	--if (PoisonerMenu:IsShown()) then
		PoisonerMenu:Hide();
	--end
	PoisonerMinimapButton:Hide();
	Poisoner_ReapplyButton:Hide();
end

function Poisoner_Enable()
	POISONER_CONFIG.Enabled = 1;
	PoisonerMinimapButton:Show();
	if (POISONER_CONFIG.StartedOnce == 0) then
		StaticPopup_Show("POISONER_STARTED_ONCE");
	end
	Poisoner_CheckReapplyButton()
end

function PoisonerMenu_Toggle()
	if (PoisonerMenu:IsShown()) then
		PoisonerMenu:Hide();
	else
		PoisonerMenu:Show();
	end
end

--
-- Dragging
--

-- Start dragging the poison button
function Poisoner_OnDragStart(self)
	if (POISONER_CONFIG.MinimapButton.Lock == 0 and not Poisoner_BeingDragged) then
		self:StartMoving();
		Poisoner_BeingDragged = true;
	else
		DEFAULT_CHAT_FRAME:AddMessage(POISONER_DRAGERROR, 1.0, 1.0, 0.5);
	end
end

-- Stop dragging the poison button
function Poisoner_OnDragStop(self)
	if (Poisoner_BeingDragged) then
		self:StopMovingOrSizing()
		Poisoner_BeingDragged = false;
	end
end

function Poisoner_DragLock()
	POISONER_CONFIG.MinimapButton.Lock = 1;
end

function Poisoner_DragUnlock()
	POISONER_CONFIG.MinimapButton.Lock = 0;
end

function Poisoner_DragReset()
	PoisonerMinimapButton:ClearAllPoints();
	PoisonerMinimapButton:SetPoint("BOTTOM", Minimap, -72, 112);
end


-- 
-- Scaling and position options
--

function Poisoner_SetScale(scale)
	POISONER_CONFIG.Scale = scale;
	PoisonerMenu:SetScale(scale);
end

function Poisoner_SetPosition(position)
	if (position == "left" or position == "right") then
		POISONER_CONFIG.MinimapButton.MenuPosition = position;
		PoisonerMenu:ClearAllPoints();
		local p, rp;
		if (position == "right") then
			p = "TOPLEFT";
			rp = "RIGHT";
		else
			p = "TOPRIGHT";
			rp = "LEFT";
		end
		PoisonerMenu:SetPoint(p, PoisonerMinimapButton, rp);
	else
		-- error
		DEFAULT_CHAT_FRAME:AddMessage(POISONER_POSITIONERROR, 1.0, 1.0, 0.5);
	end
end


--
-- ToolTips for type "full"
--

function Poisoner_Tooltip(self,pbagid,pbagslot,pttip)
	GameTooltip:SetOwner(self,"BOTTOMLEFT");
	if (POISONER_CONFIG.TooltipType == "full") then
		GameTooltip:SetBagItem(pbagid,pbagslot);
	else
		GameTooltip:SetText(pttip, 1.00, 1.00, 1.00);
	end
	GameTooltip:Show();
end


--
-- ToolTips for ReapplyButton
--

function Poisoner_ReapplyButton_Tooltip(self)
	GameTooltip:SetOwner(self,"BOTTOMLEFT");
	GameTooltip:ClearLines();
	--GameTooltip:AddLine(Poisoner_ReapplyButton_Text, 1.00, 1.00, 1.00);
	--GameTooltip:SetText(Poisoner_ReapplyButton_Text, 1.00, 1.00, 1.00);
	--GameTooltip:AddLine("|T"..Texture..":0|t ")
	GameTooltip:AddDoubleLine(Poisoner_ReapplyButton_Text_Slots, Poisoner_ReapplyButton_Text_Textures, 1,1,1, 1,1,1);
	GameTooltip:Show();
end


--
-- ReapplyButton DragHandler
--
function Poisoner_ReapplyButton_CreateOverlay()
	--	Overlay
	--[[local f = CreateFrame("Frame", "Poisoner_ReapplyButton_Overlay", UIParent)
	f:SetFrameStrata("DIALOG")
	f:SetWidth(30) 
	f:SetHeight(30)]]
	
	local t = Poisoner_ReapplyButton_Overlay:CreateTexture("Poisoner_ReapplyButton_OverlayBackground","BACKGROUND")
	t:SetTexture(1,1,1,0)
	t:SetAllPoints(Poisoner_ReapplyButton_Overlay)
	Poisoner_ReapplyButton_Overlay.texture = t
	
	Poisoner_ReapplyButton_Overlay:Show()
end

function Poisoner_ReapplyButton_UpdateLock()
	
	if (POISONER_CONFIG.ReapplyButton.Lock == 1) then
		Poisoner_ReapplyButton_Overlay:RegisterForDrag()
		Poisoner_ReapplyButton_OverlayBackground:SetTexture(0,1,0,0)
		Poisoner_ReapplyButton_Overlay:SetMovable(false)
		Poisoner_ReapplyButton_Overlay:EnableMouse(false)
		Poisoner_ReapplyButton_Overlay:SetScript("OnDragStart", nil)
		Poisoner_ReapplyButton_Overlay:SetScript("OnDragStop", nil)		
	else
		Poisoner_ReapplyButton_Overlay:Show()
		Poisoner_ReapplyButton_Overlay:RegisterForDrag("LeftButton")
		Poisoner_ReapplyButton_OverlayBackground:SetTexture(0,1,0,0.5)
		Poisoner_ReapplyButton_Overlay:SetMovable(true)
		Poisoner_ReapplyButton_Overlay:EnableMouse(true)
		Poisoner_ReapplyButton_Overlay:SetScript("OnDragStart", Poisoner_ReapplyButton_OnDragStart)
		Poisoner_ReapplyButton_Overlay:SetScript("OnDragStop", Poisoner_ReapplyButton_OnDragStop)
	end
end


-- Start dragging ReapplyButton
function Poisoner_ReapplyButton_OnDragStart(self)
	
	self:StartMoving();
	Poisoner_ReapplyButton_BeingDragged = true;
	
end

-- Stop dragging ReapplyButton
function Poisoner_ReapplyButton_OnDragStop(self)
	if (Poisoner_ReapplyButton_BeingDragged) then
		self:StopMovingOrSizing()
		POISONER_CONFIG.ReapplyButton.Position.Anchor, _, POISONER_CONFIG.ReapplyButton.Position.RelativeTo, POISONER_CONFIG.ReapplyButton.Position.XPos, POISONER_CONFIG.ReapplyButton.Position.YPos = Poisoner_ReapplyButton_Overlay:GetPoint();
		Poisoner_ReapplyButton_BeingDragged = false;
	end
end


--
-- Debugging : set to true if ya want it on
--

function PoisonerDebug(message)
	if (false) then
		if (message == nil) then
			message = "<nil>";
		end
		DEFAULT_CHAT_FRAME:AddMessage(message);
	end
end
